# Copyright (c) 2000-2006 ActiveState Software Inc.
# See the file LICENSE.txt for licensing information.

"Wrapper around nsIScriptError, so errors can be logged in the JS console"

from xpcom import _xpcom, components
import xpcom
import sys

class koConsoleMessageFactory:
    _com_interfaces_ = [components.interfaces.koIConsoleMessageFactory]
    _reg_desc_ = "Console Message Factory"
    _reg_contractid_ = "@activestate.com/koConsoleMessageFactory;1"
    _reg_clsid_ = "{F7070977-C744-42bd-8841-AA8849AC744F}"

    def newMessage(self):
        return koConsoleMessage()

class koConsoleMessage:
    _com_interfaces_ = [components.interfaces.nsIScriptError]
    _reg_desc_ = "Console Message"
    _reg_contractid_ = "@activestate.com/koConsoleMessage;1"
    _reg_clsid_ = "{A027B738-6CA8-4882-A2A7-D668A070E6FA}"

    def init(self, message, fname, line, lineNo, colNo, flags, category):
        self.message = message
        self.sourceName = fname
        self.sourceLine = line
        self.lineNumber = lineNo
        self.columnNumber = colNo
        self.flags = flags
        self.category = category

    def toString(self):
        return self.message + ' at ' + self.sourceName + ' @ ' + str(self.lineNumber)

class koConsoleListener:
    _com_interfaces_ = [components.interfaces.nsIConsoleListener]
    _reg_desc_ = "Console Listener"
    _reg_contractid_ = "@activestate.com/koConsoleListener;1"
    _reg_clsid_ = "{09D656F9-76A4-48f5-B631-A4B34433A1AB}"

    def __init__(self):
        self.output = sys.stdout

    def observe(self, message):
        try:
            message = message.QueryInterface(components.interfaces.nsIScriptError)
            flag, category, fname, lineNo, message = message.flags, message.category,\
                                                     message.sourceName, message.lineNumber,\
                                                     message.message
            
            if flag == components.interfaces.nsIScriptError.warningFlag:
                tag = 'Warning'
            elif flag == components.interfaces.nsIScriptError.errorFlag:
                tag = 'Error'
            else:
                tag = 'Debug'
            msg = u"Komodo: %(tag)s: %(fname)s(%(lineNo)d): "\
                   "%(message)s" % locals()
            msg = msg.encode("unicode-escape")
            print >>self.output, msg
        except AttributeError:
            try:
                print >>self.output, "Komodo: %s" % message.message
            except Exception:
                print >>self.output, "Komodo: No Message"
        except Exception:
            try:
                print >>self.output, "Komodo: %s" % message.message
            except Exception:
                print >>self.output, "Komodo: No Message"
        self.output.flush()
