# Copyright (c) 2000-2006 ActiveState Software Inc.
# See the file LICENSE.txt for licensing information.

# An implementation of a file object for Mozilla/xpcom.

from xpcom import components, nsError, ServerException, COMException
from xpcom.server import WrapObject
import logging
import os, os.path

log = logging.getLogger('koModelineAutocomplete')
#log.setLevel(logging.DEBUG)

class koModelineAutocomplete:
    if 0: #XXX need to port this to Firefox/XRE (no more nsIAutoCompleteSession)
        _com_interfaces_ = [components.interfaces.nsIAutoCompleteSession]
    else:
        _com_interfaces_ = [components.interfaces.nsISupports]
    _reg_desc_ = "Modeline Autocomplete Session"
    _reg_contractid_ = "@activestate.com/koModelineAutocomplete;1"
    _reg_clsid_ = "{3ACE1955-A157-4FC1-80E9-F2B1FFBC50A0}"

    def __init__(self):
        raise NotImplementedError("update koModelineAutocomplete for "
            "Firefox, which has no nsIAutoCompleteSession")

        self._viewManagerService = components.classes['@activestate.com/koViewService;1'].\
                                   getService(components.interfaces.koIViewService)
        self._windowMediator = components.classes["@mozilla.org/appshell/window-mediator;1"].\
                        getService(components.interfaces.nsIWindowMediator)
        self._focused_window = self._windowMediator.getMostRecentWindow(None)
        self._document = self._focused_window.document
        self._dialogSvc = components.classes['@activestate.com/asDialogProxy;1'].\
                          getService(components.interfaces.asIDialogProxy)
        
        self.curdirs = {}
        self.curdir = None
        self.curfiles = None
        log.info("creating koModelineAutocomplete")

    def onStartLookup(self, value, prevResult, listener):
        listener = listener.QueryInterface(components.interfaces.nsIAutoCompleteListener)
        log.info("onStartLookup %s %s %s " % (value, prevResult, listener))
        if (not value.startswith('o ') and
            not value.startswith('open ')):
            return
        if value.startswith('o '):
            value = u'open ' + value[2:]
            # XXX grrr.  I can't do this in Python, and I don't know why =(
            self._dialogSvc.eval_(self._focused_window,
                                  "document.getElementById('modeline').value = '" + value + "'")
            #m = self._document.getElementById("modeline").QueryInterface(components.interfaces.nsIDOMXULElement)
            #m.setAttribute('value', value)
        words = value.split()
        prefix = ''
        suffix = ''
        if len(words) > 1:
            prefix = words[0]
            log.info('prefix = ' + prefix)
            suffix = value.split(None, 1)[1]
        log.info("suffix = " + suffix)
        currentView = self._viewManagerService.currentView
        if (self._viewManagerService.currentView and
            self._viewManagerService.currentView.document and
            self._viewManagerService.currentView.document.file):
            dirname = self._viewManagerService.currentView.document.file.dirName;
            log.info("got dir from view" +  dirname);
        else:
            log.info("XXX got no dir from view");
            dirname = '.'
        lastSlash = max(suffix.rfind('\\'), suffix.rfind('/'))
        
        if lastSlash != -1 and \
           lastSlash != len(suffix):
            if suffix.startswith('/') or \
               suffix[1:3] == ':\\' or \
               suffix.startswith('\\\\'):
                if lastSlash > 2:
                    dirname = suffix[:lastSlash]
                else:
                    dirname = suffix[:lastSlash + 1]   
            else:
                dirname = os.path.join(dirname, suffix[:lastSlash])


        log.info("dirname = " + dirname)
        found = 0;
        if not found:
            self.curdirs[dirname] = self.curfiles = os.listdir(dirname)
        self.curdir = dirname
        lastpart = suffix[lastSlash+1:]

        results = components.classes["@mozilla.org/autocomplete/results;1"].\
                  createInstance(components.interfaces.nsIAutoCompleteResults)
        results.searchString = unicode(value)
        results.defaultItemIndex = 0
        results.param = None

        for path in self.curfiles:
            path = unicode(path)
            if path.startswith(lastpart):
                item = components.classes["@mozilla.org/autocomplete/item;1"].\
                       createInstance(components.interfaces.nsIAutoCompleteItem)
                path = suffix[0:lastSlash+1] + path
                if os.path.isdir(path):
                    item.comment = "Directory";
                item.value = u'open ' + path
                item.className = u""
                item.param = None
                results.items.AppendElement(item)
        listener.onAutoComplete(results, components.interfaces.nsIAutoCompleteStatus.matchFound)

    def onStopLookup(self):
        log.info("onStopLookup")
    
    def onAutoComplete(self, value, prevResult, listener):
        log.info("onAutoComplete %s %s %s " % (value, prevResult, listener))
        value = value[len('open '):]
        # Should do an absolute path from the current view's dirName
        currentView = self._viewManagerService.currentView
        if currentView and \
           currentView.document and \
           currentView.document.file and \
           currentView.document.file.exists:
            dir = currentView.document.file.dirName
        else:
            dir = '.'
        path = os.path.realpath(os.path.join(dir, value));
        file = components.classes['@activestate.com/koFileEx;1'].createInstance()
        file.path = path
        URI = file.URI
        self._viewManagerService.newViewFromURI(URI, 'editor')
        self._dialogSvc.eval_("document.getElementById('modeline').value = ''")
