# Copyright (c) 2000-2006 ActiveState Software Inc.
# See the file LICENSE.txt for licensing information.

# An implementation of a ftp file objects
#
# Contributors:
# * Shane Caraveo
# * Aaron Bingham

from xpcom import file, components, COMException, ServerException, server, nsError, _xpcom
import sys
import types
import gc
import koLogInstances

class koPythonGarbage:
    _com_interfaces_ = [components.interfaces.koIPythonGarbage]
    _reg_desc_ = "Python Garbage Collection Debugging Utilities"
    _reg_contractid_ = "@activestate.com/koPythonGarbage;1"
    _reg_clsid_ = "{7D29D287-A208-45e0-A67C-5DFB670300E3}"

    def __init__(self):
        self._lastGarbageLen = 0
        self._tracked = {}
        self._tracked_instances = {}

    # get reference counts for all classes
    def _getRefcounts(self):
        gc.collect()
        d = {}
        sys.modules
        # collect all classes
        for m in sys.modules.values():
            for sym in dir(m):
                o = getattr (m, sym)
                if type(o) is types.ClassType:
                    d[o.__name__] = sys.getrefcount (o)
        try:
            d["Total Python References"] = sys.gettotalrefcount()
        except AttributeError: # Only on debug builds
            pass
        d["Total Python XPCOM client interfaces"] = _xpcom._GetInterfaceCount()
        d["Total Python XPCOM gateway interfaces"] = _xpcom._GetGatewayCount()
        return d

    # print x number of reference listings
    def _doPrintRefCounts(self, d, count, category, bLog = 1):
        # sort by refcount
        pairs = map (lambda x: (x[1],x[0]), d.items())
        pairs.sort()
        pairs.reverse()
        for n, ob in pairs[:count]:
            out = '%10d %s' % (n, ob)
            if bLog:
                fileLoggingSvc = components.classes['@activestate.com/koFileLoggingService;1'].\
                            getService(components.interfaces.koIFileLoggingService);
                fileLoggingSvc.LogMeasurement("garbage", category, out);
            print out

    def printRefCounts(self, count):
        d = self._getRefcounts()
        self._doPrintRefCounts(d, count, "pyrefcount")
        koLogInstances.koReportLoggedInstances('*')

    def resetTrackedRefCounts(self):
        self._tracked = self._getRefcounts()
        self._tracked_instances = koLogInstances.tracked_classes.copy()

    def printTrackedRefCounts(self, count):
        gc.collect()
        try:
            lost_pyrefs = sys.gettotalrefcount() - self.total_refs_before
        except AttributeError: # sys.gettotalrefcount only exists in debug builds.
            lost_pyrefs = 0
        new = {}
        d = self._getRefcounts()
        for k, refs in d.items():
            b4 = self._tracked.get(k,0)
            if refs > b4:
                new[k] = refs - b4
        self._doPrintRefCounts(new, count, "pyrefcount_tracked", 0)
        print "New Python instances"
        for class_name, current_weaks in koLogInstances.tracked_classes.items():
            old_weaks = self._tracked_instances.get(class_name, [])
            old_obs = map( lambda o: o(), old_weaks )
            for weak in current_weaks:
                ob = weak()
                if ob not in old_obs:
                    print ob
            

    def gcDebugFlags(self, flags):
        gc.set_debug(eval(flags))
        
    def gcCollect(self):
        fileLoggingSvc = components.classes['@activestate.com/koFileLoggingService;1'].\
                    getService(components.interfaces.koIFileLoggingService);
        print "-"*50
        unreachable = gc.collect()
        out = "unreachable: %s" % (unreachable)
        fileLoggingSvc.LogMeasurement("garbage", "collect", out);
        print "-"*50
        garbage = gc.garbage[self._lastGarbageLen:]
        for entry in garbage:
            fileLoggingSvc.LogMeasurement("garbage", "collect", entry);
        print garbage
        self._lastGarbageLen = len(gc.garbage)
        print "-"*50

