import re

class recollector:
    def __init__(self):
        self.res = {}
        self.regs = {}
        
    def add(self, name, reg, mods=None ):
        self.regs[name] = reg % self.regs
        #print "%s = %s" % (name, self.regs[name])
        if mods:
            self.res[name] = re.compile(self.regs[name], mods) # check that it is valid
        else:
            self.res[name] = re.compile(self.regs[name]) # check that it is valid

collector = recollector()
a = collector.add

a("TextSE" , "[^<]+")
a("UntilHyphen" , "[^-]*-")
a("Until2Hyphens" , "%(UntilHyphen)s(?:[^-]%(UntilHyphen)s)*-")
a("CommentCE" , "%(Until2Hyphens)s>?") 
a("UntilRSBs" , "[^\\]]*](?:[^\\]]+])*]+")
a("CDATA_CE" , "%(UntilRSBs)s(?:[^\\]>]%(UntilRSBs)s)*>" )
a("S" , "[ \\n\\t\\r]+")
a("NameStrt" , "[A-Za-z_:]|[^\\x00-\\x7F]")
a("NameChar" , "[A-Za-z0-9_:.-]|[^\\x00-\\x7F]")
a("Name" , "(?:%(NameStrt)s)(?:%(NameChar)s)*")
a("QuoteSE" , "\"[^\"]*\"|'[^']*'")
a("DT_IdentSE" , "%(S)s%(Name)s(?:%(S)s(?:%(Name)s|%(QuoteSE)s))*" )

# http://bugs.activestate.com/show_bug.cgi?id=28765
#a("MarkupDeclCE" , "(?:[^\\]\"'><]+|%(QuoteSE)s)*>" )
a("MarkupDeclCE" , "(?:[^\\]\"'> \\n\\t\\r<]+|%(QuoteSE)s)*>" )

a("S1" , "[\\n\\r\\t ]")
a("UntilQMs" , "[^?]*\\?+")
a("PI_Tail" , "\\?>|%(S1)s%(UntilQMs)s(?:[^>?]%(UntilQMs)s)*>" )
a("DT_ItemSE" ,
    "<(?:!(?:--%(Until2Hyphens)s>|[^-]%(MarkupDeclCE)s)|\\?%(Name)s(?:%(PI_Tail)s))|%%%(Name)s;|%(S)s"
)
a("DocTypeCE" ,
"%(DT_IdentSE)s(?:%(S)s)?(?:\\[(?:%(DT_ItemSE)s)*](?:%(S)s)?)?>?" )
a("DeclCE" ,
    "--(?:%(CommentCE)s)?|\\[CDATA\\[(?:%(CDATA_CE)s)?|DOCTYPE(?:%(DocTypeCE)s)?")
a("PI_CE" , "%(Name)s(?:%(PI_Tail)s)?")
a("EndTagCE" , "%(Name)s(?:%(S)s)?>?")
a("AttValSE" , "\"[^<\"]*\"|'[^<']*'")
a("ElemTagCE" ,
    "%(Name)s(?:%(S)s%(Name)s(?:%(S)s)?=(?:%(S)s)?(?:%(AttValSE)s))*(?:%(S)s)?/?>?")

a("MarkupSPE" ,
    "<(?:!(?:%(DeclCE)s)?|\\?(?:%(PI_CE)s)?|/(?:%(EndTagCE)s)?|(?:%(ElemTagCE)s)?)")
a("XML_SPE" , "%(TextSE)s|%(MarkupSPE)s")
a("XML_MARKUP_ONLY_SPE" , "%(MarkupSPE)s")

a("DOCTYPE",        r'<!DOCTYPE\s+(?P<type>\S+)\s+(?P<ident>PUBLIC|SYSTEM)\s+(?P<data1>%(QuoteSE)s)\s*(?P<data2>%(QuoteSE)s)?\s*(?:\[|>)', re.S)

def lexxml(data, markuponly=0, matchGroups=0):
    if markuponly:
        reg = "XML_MARKUP_ONLY_SPE"
    else:
        reg = "XML_SPE"
    regex = collector.res[reg]
    if matchGroups:
        start = 0
        matchObjs = []
        while start <= len(data):
            matchObj = regex.search(data, start)
            if matchObj:
                matchObjs.append(matchObj)
                if matchObj.end() - matchObj.start():
                    start = matchObj.end()
                else: # match is zero length, advance at least one
                    start = matchObj.end() + 1
            else:
                break
        return matchObjs
    else:
        return regex.findall(data)

def assertlex(data, numtokens, markuponly=0):
    if len(lexxml(data, markuponly))!=numtokens:
        assert len(lexxml(data))==numtokens, "data = '%s', numtokens = '%s'" %(data, numotkens)

def testlexer():
    # this test suite could be larger!
    assertlex("<abc/>", 1)
    assertlex("<abc><def/></abc>", 3)
    assertlex("<abc>Blah</abc>", 3)
    assertlex("<abc>Blah</abc>", 2, markuponly=1)
    assertlex("<?xml version='1.0'?><abc>Blah</abc>", 3, markuponly=1)
    assertlex("<abc>Blah&foo;Blah</abc>", 3)
    assertlex("<abc>Blah&foo;Blah</abc>", 2, markuponly=1)
    assertlex("<abc><abc>", 2)
    assertlex("</abc></abc>", 2)
    assertlex("<abc></def></abc>", 3)

def strip_quotes(str):
    if not str:
        return None
    if str[0] in ["'",'"']:
        return str[1:-1]
    return str

PARENT, TEXT, CHILDREN, STARTTAG, ENDTAG, TEXTNODE = 0, 1, 2, 3, 4, 5
def parent(node):
    "parent of this node"
    return node[PARENT]

def text(node):
    return node[TEXT]

def children(node):
    return node[CHILDREN]

# XXX this should realy be done via DTD/Schema, but that would be a major
# pain.  For general purposes, this will work fine and be faster

# these tags are defined to NOT ALLOW end tags at all in html.  They never
# have children and never have end tags
# defined in dtd as ELEMENT NAME - O EMPTY
html_no_close_tags = [
    "basefont", "br", "area", "link", "img", "param", "hr", "input",
    "col", "frame", "isindex", "base", "meta"
]
# defined in dtd as ELEMENT NAME - O *
html_optional_close_tags = [
    "p", "dt", "dd", "li", "option", "thead", "tfoot", "colgroup",
    "col", "tr", "th", "td"
]

# The basic data structure used in here is a tuple.
# An API to it is defined above but in the code below I don't
#   use the API 
class fastxmltree:
    def __init__(self, data, prune=1, 
        PARENT=PARENT, TEXT=TEXT, CHILDREN=CHILDREN):
        # the above line is an efficiency hack
        target = dummy = current = (None, "#root", [])

        # we keep a dictionary of tags available for intellisense
        # purposes
        tags = {}
        nodes = []
        no_close_tag = []
        opt_close_tag = []
        self.docType = ""
        self.publicId = ""
        self.systemId = ""
        last_tagname = None
        
        for x in lexxml(data, markuponly=1):
            if x.startswith("<!") or x.startswith("<?"):
                if x.startswith("<!DOCTYPE"):
                    # save the document typeinformation
                    m = collector.res["DOCTYPE"].match(x)
                    data = m.groupdict()
                    self.docType = data['type'].lower()
                    if data['ident'] == "PUBLIC":
                        self.publicId = strip_quotes(data['data1'])
                        self.systemId = strip_quotes(data['data2'])
                    else:
                        self.systemId = strip_quotes(data['data1'])
                    if self.docType == "html":
                        # we want to handle tags that have no close tag
                        no_close_tag = html_no_close_tags
                        opt_close_tag = html_optional_close_tags
            elif x.startswith("</"):
                recovery = current
                mytagname = parse_etagname(x)
                # walk up the tree looking for a matching start-tag
                # do this to try to recover from "extra" start-
                while current[PARENT] and current[PARENT] is not dummy and tagname(current)!=mytagname:
                    current = current[PARENT]
    
                # if I never found a matching start-tag, revert back
                # and pretend this end-tag never happened
                if mytagname!=tagname(current):
                    current = recovery
                else:
                    # save memory by deleting nodes we aren't interested in.
                    # if we've seen an end-tag, this isn't a "live" node
                    # anymore, so its children aren't interesting to us.
                    if prune:
                        # clear the children array
                        del current[CHILDREN][:]
                    # try to go back up the stack
                    current = current[PARENT]
                    target = current
            elif x.startswith("<"):
                newnode = (current, x, [])
                mytagname = tagname(newnode)
                if opt_close_tag and mytagname in opt_close_tag:
                    # XXX not quite right yet
                    
                    # if we hit the same tag with optional close tag,
                    # we need to close the last tag
                    if mytagname == tagname(current[PARENT]):
                        if prune:
                            # clear the children array
                            del current[CHILDREN][:]
                        # try to go back up the stack
                        current = current[PARENT]
                        target = current
                current[CHILDREN].append(newnode)
                last_tagname = mytagname
                tags[mytagname] = None
                nodes.append(newnode)
                if not x.endswith("/>"):
                    if no_close_tag and mytagname in no_close_tag:
                        # we have to check if this node is one that does not
                        # use a closing tag.
                        continue
                    target = current = newnode

        # did I find any elements at all?
        if len(children(dummy))>0:
            root = children(dummy)[-1]
        else:
            target = root = None
        self.root = root
        if target and target[TEXT]=="#root":
            self.current = None
        else:
            self.current  = target
        try:
            # sometimes we get an empty tag if the input is not
            # well-formed
            del tags[""]
        except KeyError:
            pass
        self.tags = tags.keys()
        self.nodes = nodes


class xmltree:
    def __init__(self, data, prune=1, 
        PARENT=PARENT, TEXT=TEXT, CHILDREN=CHILDREN):
        # the above line is an efficiency hack
        target = dummy = current = [None, "#root", [], None, None, '']

        # we keep a dictionary of tags available for intellisense
        # purposes
        nodes = []
        no_close_tag = []
        opt_close_tag = []
        tags = {}
        self.docType = ""
        self.publicId = ""
        self.systemId = ""
        
        for matchObj in lexxml(data, markuponly=0, matchGroups=1):
            x = matchObj.group(0)
            if x.startswith("<!") or x.startswith("<?"):
                if x.startswith("<!DOCTYPE"):
                    # save the document typeinformation
                    m = collector.res["DOCTYPE"].match(x)
                    data = m.groupdict()
                    self.docType = data['type'].lower()
                    if data['ident'] == "PUBLIC":
                        self.publicId = strip_quotes(data['data1'])
                        self.systemId = strip_quotes(data['data2'])
                    else:
                        self.systemId = strip_quotes(data['data1'])
                    if self.docType == "html":
                        # we want to handle tags that have no close tag
                        no_close_tag = html_no_close_tags
                        opt_close_tag = html_optional_close_tags
            elif x.startswith("</"):
                recovery = current
                mytagname = parse_etagname(x)
                # walk up the tree looking for a matching start-tag
                # do this to try to recover from "extra" start-
                while current[PARENT] and current[PARENT] is not dummy and tagname(current)!=mytagname:
                    current = current[PARENT]

                # if I never found a matching start-tag, revert back
                # and pretend this end-tag never happened
                if mytagname!=tagname(current):
                    current = recovery
                else:
                    # save memory by deleting nodes we aren't interested in.
                    # if we've seen an end-tag, this isn't a "live" node
                    # anymore, so its children aren't interesting to us.
                    if prune:
                        # clear the children array
                        del current[CHILDREN][:]
                    current[ENDTAG] = matchObj
                    # try to go back up the stack
                    current = current[PARENT]
                    target = current
            elif x.startswith("<"):
                newnode = [current, x, [], matchObj, None, '']
                current[CHILDREN].append(newnode)
                tags[tagname(newnode)] = None
                nodes.append(newnode)
                if not x.endswith("/>"):
                    if no_close_tag and localname(newnode) in no_close_tag:
                        # we have to check if this node is one that does not
                        # use a closing tag.
                        continue
                    target = current = newnode
            else:
                current[TEXTNODE] += x.strip()

        # did I find any elements at all?
        if len(children(dummy))>0:
            root = children(dummy)[-1]
        else:
            target = root = None
        self.root = root
        if target and target[TEXT]=="#root":
            self.current = None
        else:
            self.current  = target
        try:
            # sometimes we get an empty tag if the input is not
            # well-formed
            del tags[""]
        except KeyError:
            pass
        self.tags = tags.keys()
        self.nodes = nodes

def printnode(node, index=0):
    if len(node[CHILDREN]) == 0:
        print " "*index, "<%s/>" % tagname(node)
        return
    print " "*index, "<%s>" % tagname(node)
    for i in children(node):
        printnode(i, index+2)
    print " "*index, "</%s>" % tagname(node)


def namespace(node):   
    return namespaces(node).get(prefix(node))

def tagname(node):
    if len(node[TEXT])>1:
        return collector.res["Name"].match(node[TEXT][1:]).group()
    else:
        return ""

def parse_etagname(text):
    if len(text)>2:
        return collector.res["Name"].match(text[2:]).group()
    else:
        return ""

a("attrfinderRE" , "(?:[\n \t]*)(%(Name)s)(?:%(S)s)?=(?:%(S)s)?(%(AttValSE)s)", re.S|re.U)
attrfinder = collector.res["attrfinderRE"]

def attrs(node, matchGroups=0):
    if matchGroups:
        data = text(node)
        start = 0
        matchObjs = []
        while start <= len(data):
            #print data
            matchObj = attrfinder.search(data, start)
            if matchObj:
                matchObjs.append(matchObj)
                if matchObj.end() - matchObj.start():
                    start = matchObj.end()
                else: # match is zero length, advance at least one
                    #print matchObj
                    start = matchObj.end() + 1
            else:
                break
        return matchObjs
    return attrfinder.findall(text(node))

def prefix(node):
    "return the prefix for an element"
    parts = tagname(node).split(":")
    if len(parts)==1:
        return ""
    else:
        return parts[0]
    
def localname(node):
    "return the localname for an element"
    parts = tagname(node).split(":")
    if len(parts)==1:
        return parts[0]
    else:
        return parts[1]

def qname(node):
    "return a tuple containing prefix and localname for an element"
    parts = tagname(node).split(":")
    if len(parts)==1:
        return ("", parts[0])
    else:
        return parts
    
def prefix_for(node, url):
    "find an XML namespace prefix for a given a node and a namespace URL"
    names = namespaces(node)
    for prefix, namespace in names.items():
       if url==namespace:
           return prefix
    return ""

def namespaces(node):
    "list the namespaces for a node as a dictionary"
    namespaces = {}
    while node is not None:
        for attrname, attrval in attrs(node):
            if attrname.startswith("xmlns"):
                if len(attrname)>5 and attrname[5]==":":
                    prefix = attrname[6:]
                else:
                    prefix = ""
                namespaces.setdefault(prefix, attrval[1:-1]) # strip quotes
        node = parent(node)
    return namespaces

def locator(node, endTag=0):
    if len(node) < STARTTAG:
        return (None, None)
    if endTag and node[ENDTAG]:
        return node[ENDTAG].span(0)
    return node[STARTTAG].span(0)
    
if __name__=="__main__":
    data = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">

    <html>
    <body>
    <p>
        <img>
        <p>
            <br>
        </p>
        <hr>
        <p>
            <br>
        <dl>
            <li>
            <li>
            <li>
        </dl>
    </p>
    </body>
    </html>
    """
    tree = fastxmltree(data, prune=0)
    printnode(tree.root)
    assert localname(tree.root) == "html"
    assert len(tree.root[CHILDREN]) == 1
    body = tree.root[CHILDREN][0]
    assert len (body[CHILDREN]) == 1
    par1 = body[CHILDREN][0]
    assert len (par1[CHILDREN]) == 4
    print tree.docType
    #for at in attrs(x, 1):
    #    print at, at.span(0), repr(x[TEXT][at.span(0)[0]:at.span(0)[1]])
    
#    data = """
#<xsl:foo xmlns="default ns" xmlns:xsl="xsl namespace" xmlns:blah='blah namespace'>
#   <incomplete blah="blah"
#   <xsl2:foo xmlns:xsl2="xsl namespace">
#      <bogus xmlns:something="something" 
#            xmlns:blah="not blah"
#>   </random-end-tag>
#      </bogus>
#      </random-end-tag>
#      <bogus>
#          <false-start-tag-to-test-recovery>
#              <another-false-start-tag>
#                  <a-child-element/>
#          </random-end-tag>            
#      </bogus>
#      </random-end-tag>
#
#      <spam xmlns:xsl="another xsl namespace"> <bogus1/> <bogus2> </bogus2> abc 
#"""
#    testlexer()
#    import sys
#    import time
#    if len(sys.argv)>1:
#        data = open(sys.argv[1]).read()
#        start = time.time()
#        tree = fastxmltree(data)
#        ns = namespaces(tree.current)
#    else:
#        tree = fastxmltree(data)
#        root, current = tree.root, tree.current
#        assert tagname(current)=="spam"
#        assert "xmlns:xsl" in [x for x,y in attrs(current)]
#        assert namespaces(current)["xsl"] == "another xsl namespace"
#        assert namespaces(current)["xsl2"] == "xsl namespace"
#        assert "something" not in namespaces(current).keys()
#        assert tagname(children(parent(current))[0]) == "bogus"
#        assert tagname(current) == "spam"
#        assert namespace(current) == "default ns"
#        assert tagname(parent(current)) == "xsl2:foo"
#        assert tagname(root) == "xsl:foo"
#        assert namespace(root) == "xsl namespace"
#        assert namespaces(root)[""] == "default ns"
#        assert namespaces(root)["xsl"] == "xsl namespace"
#
#        data = """
#        <abc>
#            <def>
#                <blah """
#        tree = fastxmltree(data)
#        assert tagname(tree.current) == "blah"
#
#        data = """
#        <abc>
#            <def>
#                <blah a="foo" """
#        tree = fastxmltree(data)
#        assert tagname(tree.current) == "blah"
#
#        data = """
#        <abc>
#            <def>
#                <blah a="foo"> <"""
#        tree = fastxmltree(data)
#        assert tagname(tree.current) == ""
#
#        data = """<foo/>"""
#        tree = fastxmltree(data)
#        assert not tree.current
#        
#        data = """</unbalanced>
#        <a>"""
#        tree = fastxmltree(data)
#        assert tagname(tree.current)=="a"
#
#        data = """</unbalanced>
#        <a>
#        <b>"""
#        tree = fastxmltree(data)
#        assert tagname(tree.current)=="b"
#
#        data = """
#        <multi-root></multi-root>
#        <multi-root2></multi-root2>
#        <b>"""
#        tree = fastxmltree(data)
#        assert tagname(tree.current)=="b"
#
#        data = """</unbalanced>
#        <a></a>
#        <c></c>
#        <b>"""
#        tree = fastxmltree(data)
#        assert tagname(tree.current)=="b"
#
#        data = "</unbalanced>"
#        tree = fastxmltree(data)
#        assert not tree.current
#
#        data = "<abc></abc></abc>"
#        tree = fastxmltree(data)
#        assert not tree.current
#
#        data = "<abc></abc>"
#        tree = fastxmltree(data)
#        assert not tree.current
